/*
 * Copyright (c) 2007 Sun Microsystems, Inc.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.sunspotworld;

import com.sun.spot.sensorboard.EDemoBoard;
import com.sun.spot.sensorboard.peripheral.ISwitch;
import com.sun.spot.sensorboard.peripheral.ITriColorLED;
import com.sun.spot.io.j2me.radiostream.*;
import com.sun.spot.io.j2me.radiogram.*;
import com.sun.spot.sensorboard.io.IScalarInput;
import com.sun.spot.sensorboard.peripheral.ISwitchListener;
import com.sun.spot.util.*;

import java.io.*;
import javax.microedition.io.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

/**
 * The startApp method of this class is called by the VM to start the
 * application.
 * 
 * The manifest specifies this class as MIDlet-1, which means it will
 * be selected for execution.
 * 
 * This is the main class for the Bend Sensor application and 'controls'
 * the reading of sensor data and sending of sensor data to the remote
 * Sun SPOT.
 */
public class Bendy extends MIDlet implements ISwitchListener {

    private ITriColorLED[] leds = EDemoBoard.getInstance().getLEDs();
    private static final int SAMPLE_SIZE = 150;
    private int bendMin = 1000; // any high value will do.
    private int bendCenter = 0; // any low value will do
    private int bendMax = 0; // any low value will do
    private IScalarInput bend = EDemoBoard.getInstance().getScalarInputs()[EDemoBoard.A0]; // the bend sensor is connected to pin A0
    private ISwitch sw1 = EDemoBoard.getInstance().getSwitches()[EDemoBoard.SW1];
    private ISwitch sw2 = EDemoBoard.getInstance().getSwitches()[EDemoBoard.SW2];
    private BendySender nw;

    /**
     * This is the main method called to start the Sun SPOT Application
     * @throws javax.microedition.midlet.MIDletStateChangeException Thrown when the state of the MIDlet is changed
     */
    protected void startApp() throws MIDletStateChangeException {
        new BootloaderListener().start();   // monitor the USB (if connected) and recognize commands from host
        nw = new BendySender();
        calibrate();
        sw2.addISwitchListener(this);
        readSensor(); // this never returns.
    }

    /**
     * Reads the data from the Sensor itself.
     * 
     * This method never returns.
     */
    private void readSensor() {
        int lastVal = 0;
        int bendInt = 0;
        leds[7].setRGB(0, 250, 0);
        while (true) {
            try {
                bendInt = bend.getValue();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            if ((bendInt > lastVal + 15) || (bendInt < lastVal - 15)) {
                lastVal = bendInt;
                String msg = String.valueOf(bendInt);
                System.out.println("Read Sensor Value: " + msg);
                if (nw.isConnected()) {
                    try {
                        nw.send(msg);
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }
            }
            leds[7].setOn(!leds[7].isOn());
            Utils.sleep(50);
        }
    }

    /**
     * Calibrate (or re-calibrate) the sensor based on 
     * user input.
     */
    protected void calibrate() {
        leds[7].setOff();
        setBendMin(1000);
        setBendCenter(0);
        setBendMax(0);
        System.out.println("Do not touch bend sensor while red light is flashing ...");
        Utils.sleep(1000);
        leds[0].setRGB(250, 0, 0); // set color to moderate red
        int z = 0;
        /* 
         * Collect a large sample size and average it to find the 'neutral' position
         */
        while (z < SAMPLE_SIZE) {
            try {
                setBendCenter(getBendCenter() + bend.getValue());
                z++; // only collect successful reads
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            leds[0].setOn(!leds[0].isOn()); // Blink LED
            Utils.sleep(50);
        }
        setBendCenter(getBendCenter() / SAMPLE_SIZE); // average over the sample size
        System.out.println("Center Average: " + getBendCenter() + "(over " + SAMPLE_SIZE + " readings)");
        System.out.println("Calibrate bend Sensor. Bend in both directions several times. Click Switch one when done.");
        leds[0].setRGB(0, 0, 250); // set to blue
        while (sw1.isOpen()) {                  // done when switch is pressed
            try {
                int bendVal = bend.getValue();
                if (bendVal < getBendMin()) {
                    setBendMin(bendVal);
                }
                if (bendVal > getBendMax()) {
                    setBendMax(bendVal);
                }
                System.out.println("Bend: " + bendVal + "    Min: " + getBendMin() + "    Max: " + getBendMax());
                leds[0].setOn(!leds[0].isOn()); // Blink LED
                Utils.sleep(50);
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        System.out.println(" Setting Min: " + getBendMin() + "    Center: " + getBendCenter() + "     Max: " + getBendMax());
        leds[0].setOff();
        nw.setSenseCenter(bendCenter);
        nw.setSenseMax(bendMax);
        nw.setSenseMin(bendMin);
        // nw.sendCalibration();

    }

    protected void pauseApp() {
    // This is not currently called by the Squawk VM
    }

    /**
     * Called if the MIDlet is terminated by the system.
     * I.e. if startApp throws any exception other than MIDletStateChangeException,
     * if the isolate running the MIDlet is killed with Isolate.exit(), or
     * if VM.stopVM() is called.
     * 
     * It is not called if MIDlet.notifyDestroyed() was called.
     * @param unconditional If true when this method is called, the MIDlet must
     *    cleanup and release all resources. If false the MIDlet may throw
     *    MIDletStateChangeException  to indicate it does not want to be destroyed
     *    at this time.
     * @throws javax.microedition.midlet.MIDletStateChangeException 
     */
    protected void destroyApp(boolean unconditional) throws MIDletStateChangeException {
        for (int i = 0; i < 8; i++) {
            leds[i].setOff();
        }
    }

    /**
     * Return the currently-set minimum value for the bend sensor
     * @return Sensor minimum value
     */
    public int getBendMin() {
        return bendMin;
    }

    /**
     * Set the minimum value read from the Bend Sensor during configuration
     * @param bendMin The minimum value read from the sensor
     */
    public void setBendMin(int bendMin) {
        this.bendMin = bendMin;
    }

    /**
     * Return the currently stored 'center' point of the bend sensor
     * as calculated
     * @return The stored center-point of the sensor
     */
    public int getBendCenter() {
        return bendCenter;
    }

    /**
     * Store the current 'center' point of the bend sensor
     * as calculated
     * @param bendCenter The calculated center point
     */
    public void setBendCenter(int bendCenter) {
        this.bendCenter = bendCenter;
    }

    /**
     * Return the currently stored 'max' point of the bend sensor
     * as calculated
     * @return maximum possible sensor value
     */
    public int getBendMax() {
        return bendMax;
    }

    /**
     * set the current  'maximum' point of the bend sensor
     * as calculated
     * @param bendMax calculated maximum sensor value
     */
    public void setBendMax(int bendMax) {
        this.bendMax = bendMax;
    }

    /* 
     * We watch Switch 2 for a press, and if pressed, recalibrate the bend sensor
     * and send the new calibration data to the servo.
     */
    /**
     * Monitor Switch for a press. 
     * When pressed, re-calibrate the bend sensor and
     * send recalibration message to the remote
     * Sun SPOT.
     * @param sw The switch to watch
     */
    public void switchPressed(ISwitch sw) {
        int switchNum = (sw == sw1) ? 1 : 2;
        System.out.println("Switch " + switchNum + " closed.");
        calibrate();
    }

    public void switchReleased(ISwitch arg0) {
    }
}
