/*
* Copyright (c) 2007 Sun Microsystems, Inc.
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use, copy,
* modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
* BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
* ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/


package org.sunspotworld;

import com.sun.spot.io.j2me.radiostream.RadiostreamConnection;
import com.sun.spot.peripheral.NoRouteException;
import com.sun.spot.peripheral.Spot;
import com.sun.spot.util.IEEEAddress;
import com.sun.squawk.VM;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import javax.microedition.io.Connector;

/**
 * Main Sensor networking code for sending sensor values to the remote SPOT
 */
public class BendySender {

    private String myAddress;
    private String otherAddress;
    private long ourAddr;
    private final int PORT = 33;
    private static final int XMIT_POWER = -31; // keep it low to prevent interference
    private boolean connected = false;
    private RadiostreamConnection conn = null;
    private DataInputStream dis = null;
    private DataOutputStream dos = null;
    private Bendy sensor;
    private int senseMin;
    private int senseMax;
    private int senseCenter;

	/**
	 * Create a new Sender Object and initialize the radio
	 */
    public BendySender() {
        init();
    }
    

	/**
	 * Initialize the radio stack for connecting to the remote Sun SPOT
	 */
    public void init() {
        ourAddr = Spot.getInstance().getRadioPolicyManager().getIEEEAddress(); // we need our own address
        Spot.getInstance().getRadioPolicyManager().setOutputPower(XMIT_POWER);
        System.out.println("Our radio address = " + IEEEAddress.toDottedHex(ourAddr));
        myAddress = IEEEAddress.toDottedHex(ourAddr);
        otherAddress = VM.getManifestProperty("ServoSPOT");
        System.out.println("Other radio address = " + otherAddress);
        try {
            conn = (RadiostreamConnection) Connector.open("radiostream://" + getOtherAddress() + ":" + PORT);
            dis = (DataInputStream) conn.openDataInputStream();
            dos = (DataOutputStream) conn.openDataOutputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }



	/**
	 * Connect to the remote SPOT and send configuration data. 
	 * This method can be called at any time to re-send the 
	 * configuration data.
	 * 
	 * This method will block on a read from the remote SPOT until the remote
	 * SPOT replies with the 'GO' command indicating that the configuration
	 * data has been received.
	 * @return True if connection succeeded,  false otherwise
	 */
    public boolean sendCalibration() {
        System.out.println("Starting connection to " + getOtherAddress() + " on port " + PORT);
        connected = false;
        while (!connected) {

            try {
                System.out.println("Sending min:" + senseMin + ";center:" + senseCenter + ";max:" + senseMax);
                dos.writeUTF("min:" + senseMin + ";center:" + senseCenter + ";max:" + senseMax);
                dos.flush();
                String response = dis.readUTF();
                if (response.equalsIgnoreCase("GO")) {
                    System.out.println("Calibration complete!");
                    connected = true;
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }

        }
        return isConnected();
    }

	/**
	 * Close the connection to the remote Sun SPOT
	 */
    public void closeConnection() {
        if (isConnected() == false) {
            return;
        }
        connected = false;
        try {
            dos.close();
            dis.close();
            conn.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

	/**
	 * Send sensor or configuration data to the remote Sun SPOT.
	 * @param message The sensor or configuration data to send to the remote Sun SPOT
	 * @throws com.sun.spot.peripheral.NoRouteException Thrown if no route is found for the remote SPOT
	 * @throws java.io.IOException Thrown if the send to the remote SPOT Failed
	 */
    public void send(String message) throws IOException, NoRouteException {
        dos.writeUTF(message);
        dos.flush();
    }

    public String getAddress() {
        return myAddress;
    }

    public String getOtherAddress() {
        return otherAddress;
    }

    public int getSenseMin() {
        return senseMin;
    }

    public void setSenseMin(int senseMin) {
        this.senseMin = senseMin;
    }

    public int getSenseMax() {
        return senseMax;
    }

    public void setSenseMax(int senseMax) {
        this.senseMax = senseMax;
    }

    public int getSenseCenter() {
        return senseCenter;
    }

    public void setSenseCenter(int senseCenter) {
        this.senseCenter = senseCenter;
    }

    public boolean isConnected() {
        return connected;
    }

}
