/*
* Copyright (c) 2007 Sun Microsystems, Inc.
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use, copy,
* modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
* BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
* ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/

package org.sunspotworld;

import com.sun.spot.sensorboard.EDemoBoard;
import com.sun.spot.sensorboard.peripheral.ISwitch;
import com.sun.spot.sensorboard.peripheral.ITriColorLED;
import com.sun.spot.io.j2me.radiostream.*;
import com.sun.spot.io.j2me.radiogram.*;
import com.sun.spot.sensorboard.peripheral.ISwitchListener;
import com.sun.spot.sensorboard.peripheral.Servo;
import com.sun.spot.util.*;
import java.io.*;
import javax.microedition.io.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

/**
 * The startApp method of this class is called by the VM to start the
 * application.
 * 
 * The manifest specifies this class as MIDlet-1, which means it will
 * be selected for execution.
 * 
 * This class controls the actual Servo, and reads from another class which
 * controls the Radio.
 */
public class ServoMover extends MIDlet implements ISwitchListener{

    private ITriColorLED[] leds = EDemoBoard.getInstance().getLEDs();
    private static final int CENTER = 1500;
    private static final int MAX = 2500;
    private static final int MIN = 500;
    private ISwitch[] switches = EDemoBoard.getInstance().getSwitches();
    private ServoReceiver snw;
    private Servo serv;

	/**
	 * This is the main method called to start the MIDlet application.
	 * @throws javax.microedition.midlet.MIDletStateChangeException Throw exception if the MIDLet fails
	 */
    protected void startApp() throws MIDletStateChangeException  {
        new BootloaderListener().start();   // monitor the USB (if connected) and recognize commands from host
        switches[0].addISwitchListener(this);
        switches[1].addISwitchListener(this);
        serv = new Servo(EDemoBoard.getInstance().getOutputPins()[EDemoBoard.H0]);
        serv.setValue(CENTER);
        Utils.sleep(500);
        while (serv.getValue() < MAX) {
            int cur = serv.getValue();
            serv.setValue(cur + 100);
            Utils.sleep(250);
        }
        while (serv.getValue() > MIN) {
            int cur = serv.getValue();
            serv.setValue(cur - 100);
            Utils.sleep(250);
        }
//        snw = new ServoReceiver(this);
//        System.out.println("Waiting for calibration data...");
//        snw.connect();
//        Thread t = new Thread(snw);
//        t.start();
    }
    
	/**
	 * Move the servo the specified amount.
	 * @param bendValue Value of the sensor read. Must be converted to a servo-position before use.
	 */
    public void moveServo(int bendValue) {
        /*
         * the formula is, roughly: 1000 * cube-root[(value-min)/60] + 500, but since we
         * only do *integer* cube roots, we make the value *really* big first (10^3), then 
         * reduce it again ( /10 ) later. It's not perfect, but it's pretty close
         */
        int sMove = (icbrt((int) ((((double) bendValue - (double) snw.getMin()) / 60) * 1000000)) * 10) + 500;
        System.out.println("Read sensor value: " + bendValue + " Moving Servo to: " + sMove);
        serv.setValue(sMove);

    }
    
    /**
     * Handler called when a watched switch is pressed.
     * @param sw the switch that was pressed
     */
    public void switchPressed(ISwitch sw) {
        int switchNum = (sw == switches[0]) ? 1 : 2;
        System.out.println("Switch " + switchNum + " closed.");
        manualMove(switchNum);
    }

    /**
     * Handler called when a switch is released
     * @param sw the switch that was released
     */
    public void switchReleased(ISwitch sw) {
         int switchNum = (sw == switches[0]) ? 1 : 2;
        System.out.println("Switch " + switchNum + " opened.");
        manualMove(switchNum);
    }
    
    /**
     * Move a servo based on a value for direction
     * @param dir 1 or 2, corresponds to switch 1 or 2 to move the servo
     */
    private void manualMove (int dir) {
        final  int move = 25;
        int servPos = serv.getValue();
        if(dir == 1){
            if(servPos - move > MIN){
                serv.setValue(servPos - move);
            } else {
                serv.setValue(MIN);
            }
        } else {
            if(servPos + move < MAX ){
                serv.setValue(servPos + move);
            } else {
                serv.setValue(MAX);
            }
        }
    }


    
	/**
	 * This is ugly code, found on the internet, for doing, basically, integer cube roots.
	 * nasty stuff, but it comes astonishingly close, so it's 'good enough'
	 * Squawk doesn't have built-in cube-root, so this is our only hope.
	 * @return Integer cube-root
	 * @param x The integer to reduce to cube-root
	 */
    private int icbrt(int x) {
        int s = 30;
        int y, b;
        y = 0;
        while (s >= 0) {
            y = 2 * y;
            b = (3 * y * (y + 1) + 1) << s;
            s = s - 3;
            if (x >= b) {
                x = x - b;
                y = y + 1;
            }
        }
        return y;
    }

    protected void pauseApp() {
    // This is not currently called by the Squawk VM
    }

    /**
	 * Called if the MIDlet is terminated by the system.
	 * I.e. if startApp throws any exception other than MIDletStateChangeException,
	 * if the isolate running the MIDlet is killed with Isolate.exit(), or
	 * if VM.stopVM() is called.
	 * 
	 * It is not called if MIDlet.notifyDestroyed() was called.
	 * @param unconditional If true when this method is called, the MIDlet must
	 *    cleanup and release all resources. If false the MIDlet may throw
	 *    MIDletStateChangeException  to indicate it does not want to be destroyed
	 *    at this time.
	 * @throws javax.microedition.midlet.MIDletStateChangeException Thrown when the state of the MIDLet is changed.
	 */
    protected void destroyApp(boolean unconditional) throws MIDletStateChangeException {
        for (int i = 0; i < 8; i++) {
            leds[i].setOff();
        }
    }

    
    
}
