/*
* Copyright (c) 2007 Sun Microsystems, Inc.
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use, copy,
* modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
* BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
* ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/

package org.sunspotworld;

import com.sun.spot.io.j2me.radiostream.RadiostreamConnection;
import com.sun.spot.peripheral.NoRouteException;
import com.sun.spot.peripheral.Spot;
import com.sun.spot.peripheral.TimeoutException;
import com.sun.spot.util.IEEEAddress;
import com.sun.spot.util.Utils;
import com.sun.squawk.VM;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import javax.microedition.io.Connector;


/**
 * Main Radio code for connecting to the Remote Sending Sun SPOT and for reading sensor values from it.
 * The addresses for this SPOT and the remote (sending) SPOT should be defined in the Manifest file
 * as 'BendySPOT'(remote SPOT) and 'ServoSPOT' (this SPOT)
 */
public class ServoReceiver implements Runnable {

    private String myAddress;
    private String otherAddress;
    private final int PORT = 33;
	/**
	 * Radio power. Set to near minimum to reduce interference
	 */
    private static int XMIT_POWER = -31;
	/**
	 * Set when connected to the remote sending SPOT
	 */
    private boolean connected;
    private RadiostreamConnection conn = null;
    private DataInputStream dis = null;
    private DataOutputStream dos = null;
    private int max = 0;
    private int min = 0;
    private int center = 0;
    private ServoMover owner;

    
    
	/**
	 * Set up a new ServoReceiver
	 * @param owner The class contining the servo to be controlled
	 */
    public ServoReceiver(ServoMover owner) {
        setOwner(owner);
        init();

    }

    

	/**
	 * Initialize the radio connection
	 */
    public void init() {
        long ourAddr = Spot.getInstance().getRadioPolicyManager().getIEEEAddress();
        Spot.getInstance().getRadioPolicyManager().setOutputPower(XMIT_POWER);
        System.out.println("Our radio address = " + IEEEAddress.toDottedHex(ourAddr));
        otherAddress = VM.getManifestProperty("BendySPOT");
        System.out.println("Other radio address = " + otherAddress);
        System.out.println("Starting connection to " + getOtherAddress() + " on port " + PORT);
        try {
            conn = (RadiostreamConnection) Connector.open("radiostream://" + getOtherAddress() + ":" + PORT);     
            dis = (DataInputStream) conn.openDataInputStream();
            dos = (DataOutputStream) conn.openDataOutputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

	/**
	 * Connect to the remote Sun SPOT
	 * @return True if connection was successful, false otherwise.
	 */
    public boolean connect() {
        connected = false;
        String tmp = null;

        while (!connected) {
            try {
                tmp = dis.readUTF();
                System.out.println("Received: " + tmp);
            } catch (TimeoutException e) {
                System.out.println("Timeout... other end is not responding");
            } catch (IOException e1) {
                System.out.println("Exception on readUTF");
            }

            if (tmp != null) {
                if (tmp.startsWith("min")) {
                    parseConfig(tmp);
                    try {
                        send("GO");
                        connected = true;
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        }
        return connected;
    }
    
	/**
	 * continually read values from the network connection and send them to the Servo
	 */
    public void run() {
        
        while (true) {
            try {
                owner.moveServo(readValue());
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }
    
	/**
	 * Read the value of the remote sensor from the network connection
	 * @return Return the value read from the remote sensor
	 * @throws java.io.IOException Reading failed
	 */
    private int readValue() throws IOException {
        String val = dis.readUTF();
        if (val.startsWith("min")) {
            parseConfig(val);
            try {
                send("GO");
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            return this.getCenter();
        } else {
            return Integer.parseInt(val);
        }
    }

	/**
	 * parse the variuous parse of a configuration message into min, center and max values and set them.
	 * @param config The String containing the config message
	 */
    private void parseConfig(String config) {
        String[] vals = Utils.split(config, ';');
        this.min = Integer.parseInt(Utils.split(vals[0], ':')[1]);
        this.center = Integer.parseInt(Utils.split(vals[1], ':')[1]);
        this.max = Integer.parseInt(Utils.split(vals[2], ':')[1]);
        System.out.println("Received Configuration data ... Min: " + this.min + " Center: " + this.center + " Max: " + this.max);
    }
    
    

	/**
	 * Close the radio connection to the remote Sun SPOT
	 */
    public void closeConnection() {
        if (connected == false) {
            return;
        }
        connected = false;
        try {
            dos.close();
            dis.close();
            conn.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

	/**
	 * Send a message to the other Sun SPOT
	 * @param message The message to send to the other Sun SPOT
	 * @throws com.sun.spot.peripheral.NoRouteException Sending of the message failed because the other SPOT could not be found
	 * @throws java.io.IOException Writing of the message failed
	 */
    public void send(String message) throws IOException, NoRouteException {
        dos.writeUTF(message);
        dos.flush();
    }

   
	/**
	 * Return the address of this Sun SPOT as a String
	 * @return IEEE Address of this Sun SPOT as a String
	 */
    public String getMyAddress() {
        return myAddress;
    }

	/**
	 * Setter for my address
	 * @param myAddress IEEE Address of this Sun SPOT
	 */
    public void setMyAddress(String myAddress) {
        this.myAddress = myAddress;
    }

	/**
	 * Return the address of the sending Sun SPOT
	 * @return Sending Address as a String
	 */
    public String getOtherAddress() {
        return otherAddress;
    }

	/**
	 * The address of the Bend Sensor Sun SPOT
	 * @param otherAddress The address of the sending Sun SPOT
	 */
    public void setOtherAddress(String otherAddress) {
        this.otherAddress = otherAddress;
    }

	/**
	 * Return the maximum value that the Bend Sensor will send
	 * @return Max value that the Bend Sensor will send
	 */
    public int getMax() {
        return max;
    }

	/**
	 * the minimum value that the bend sensor will send
	 * @return Minimum bend sensor value
	 */
    public int getMin() {
        return min;
    }

	/**
	 * get the center point of the bend sensor as sent by the bend sensor
	 * @return Center point of the bend sensor
	 */
    public int getCenter() {
        return center;
    } 

	/**
	 * Set the controlling servo class
	 * @param owner The class which controls the servo itself.
	 */
    public void setOwner(ServoMover owner) {
        this.owner = owner;
    }
}
