/*
 * Copyright  2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2007 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

package com.sun.spot.dmamemory;

import java.util.Hashtable;

import com.sun.spot.peripheral.SpotFatalException;
import com.sun.squawk.VM;
import com.sun.squawk.vm.ChannelConstants;

/**
 * Manages a chunk of uncached memory to be used for DMA buffers - see {@link IDMAMemoryManager}
 */
public class DMAMemoryManager implements IDMAMemoryManager {

	private MemoryBlock freeBlocks = new MemoryBlock(0, 0, "DUMMY");
	private MemoryBlock allocatedBlocks = new MemoryBlock(0, 0, "DUMMY");

	/**
	 * For use by the SPOT library only - to get a handle to a DMA Memory Manager use
	 * Spot.getInstance().getDMAMemoryManager()
	 */
	public DMAMemoryManager() {
		int size = VM.execSyncIO(ChannelConstants.GET_DMA_BUFFER_SIZE, 0);
		int startAddress = VM.execSyncIO(ChannelConstants.GET_DMA_BUFFER_ADDRESS, 0);
		freeBlocks.next = new MemoryBlock(startAddress, size, "");
	}

	public synchronized int getBuffer(int size, String comment) throws NotEnoughDMAMemoryException {
		if (size <= 0) throw new SpotFatalException("Request for DMA buffer of size " + size);
		MemoryBlock block = freeBlocks;
		while (block.next != null) {
			if (block.next.size < size) {
				// too small, move on to next
				block = block.next;
			} else {
				int result = block.next.startAddress;
				block.next.comment = comment;
				block.next.allocate(block, size);
				return result;
			}
		}
		throw new NotEnoughDMAMemoryException("Cannot allocate DMA buffer of size " + size + ", biggest available is " + getMaxAvailableBufferSize());
	}

	public synchronized int getMaxAvailableBufferSize() {
		MemoryBlock b = freeBlocks;
		while (b.next != null) {
			b = b.next;
		}
		return b.size;
	}

	public synchronized void releaseBuffer(int memAddr) {
		MemoryBlock allocatedBlock = removeBlock(allocatedBlocks, memAddr);
		if (allocatedBlock == null) {
			throw new SpotFatalException("Attempt to release unallocated DMA buffer at address " + memAddr);
		}
		insertFreeBlock(allocatedBlock);
	}
	
	public synchronized Hashtable getAllocationDetails() {
		Hashtable result = new Hashtable();
		MemoryBlock block = allocatedBlocks.next;
		while (block != null) {
			result.put(block.comment, new Integer(block.size));
			block = block.next;
		}
		return result;
	}
	
	private MemoryBlock removeBlock(MemoryBlock root, int memAddr) {
		MemoryBlock block = root;
		while (block.next != null) {
			if (block.next.startAddress == memAddr) {
				MemoryBlock result = block.next;
				block.next = block.next.next;
				return result;
			}
			block = block.next;
		}
		return null;
	}

	private void insertFreeBlock(MemoryBlock block) {
		MemoryBlock b = removeBlock(freeBlocks, block.startAddress-block.size);
		if (b != null) {
			// the block of memory just before this one was free - merge them
			b.size = b.size + block.size;
			insertFreeBlock(b);
			return;
		}
		b = removeBlock(freeBlocks, block.startAddress+block.size);
		if (b != null) {
			// the block of memory just after this one was free - merge them
			block.size = b.size + block.size;
		}
		b = freeBlocks;
		while (b.next != null) {
			if (b.next.size < block.size) {
				b = b.next;
			} else {
				break;
			}
		}
		block.next = b.next;
		b.next = block;
	}

	private class MemoryBlock {
		MemoryBlock next = null;
		int startAddress;
		int size;
		String comment;

		public MemoryBlock(int startAddress, int size, String comment) {
			this.startAddress = startAddress;
			this.size = size;
			this.comment = comment;
		}

		/*
		 * Allocate all or part of this block
		 */
		public void allocate(MemoryBlock previousBlock, int requestedSize) {
			previousBlock.next = next;
			next = allocatedBlocks.next;
			allocatedBlocks.next = this;
			if (size != requestedSize) {
				insertFreeBlock(new MemoryBlock(startAddress+requestedSize, size-requestedSize, ""));
			}
			size = requestedSize;
		}
	}
}
