/*
 * Copyright  2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

package com.sun.spot.flashmanagement;

import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;

/**
 * FlashFileInputStream represents an input stream over some flash memory. There are two uses for this.<br /> 
 * <br />
 * The first is to stream over the flash memory allocated to an instance of a {@link com.sun.spot.flashmanagement.FlashFile}.
 * In this case, you should construct the input stream accordingly, for example:<br />
 * <p><blockquote><pre>
 * FlashFileInputStream ffis = new FlashFileInputStream(new FlashFile("my file"));<br />
 * </pre></blockquote></p>
 * In this case, the input stream will signal end of file when the reader reaches the end of the 
 * data that has been written to the FlashFile.<br />
 * <br />
 * The second use of FlashFileInputStream is to stream over the flash memory in an arbitrary sector of the 
 * flash memory. In this case, construct the input stream like this:<br />
 * <p><blockquote><pre>
 * FlashFileInputStream ffis = new FlashFileInputStream(new NorFlashSector(mySectorNumber));<br />
 * </pre></blockquote></p>
 * In this case, the input stream will signal end of file when the reader reaches the end of the
 * physical sector.
 */
public class FlashFileInputStream extends InputStream {

	private byte[] singleByteBuffer = new byte[1];
	private int offsetInSector;
	private IAddressableNorFlashSector currentSector;
	private FlashFileDescriptor fileDescriptor;
	private int offsetInFile;

	/**
	 * Construct an input stream over the data previously written to a FlashFile 
	 * @param file The FlashFile to read data from
	 * @throws IOException If the file does not exist
	 */
	public FlashFileInputStream(FlashFile file) throws IOException {
		this(file.getFileDescriptor());
	}
		
	/**
	 * Construct a input stream over the raw data in a sector of the flash memory
	 * @param sector The sector to read data from
	 */
	public FlashFileInputStream(final IAddressableNorFlashSector sector) {
		this(new FlashFileDescriptor(
				"temp",
				sector.getSize(),
				new Vector(1) {{
					addElement(sector);
				}},
				0, // virtual address is ignored for this usage
				0,
				"",
				(short)0));

	}
	
	FlashFileInputStream(FlashFileDescriptor flashFileDescriptor) {
		fileDescriptor = flashFileDescriptor;
		offsetInFile = offsetInSector = 0;
		currentSector = fileDescriptor.getFirstSector();
	}

	/**
	 * @see java.io.InputStream#read(byte[], int, int)
	 */
	public int read(byte[] b, int off, int len) throws IOException {
		int numberOfBytesToReturn = Math.min(len, fileDescriptor.length()-offsetInFile);
		if (numberOfBytesToReturn == 0) {
			//EOF
			return -1;
		}
		int numberOfBytesToRead = numberOfBytesToReturn;
		while (numberOfBytesToRead > 0) {
			if (offsetInSector == currentSector.getSize()) {
				currentSector = fileDescriptor.getNextSector(currentSector);
				offsetInSector = 0;
			}
			int lenToReadFromThisSector = Math.min(numberOfBytesToRead, currentSector.getSize()-offsetInSector);
			currentSector.getBytes(offsetInSector, b, off, lenToReadFromThisSector);
			off += lenToReadFromThisSector;
			numberOfBytesToRead -= lenToReadFromThisSector;
			offsetInSector += lenToReadFromThisSector;
			offsetInFile += lenToReadFromThisSector;
		}
		return numberOfBytesToReturn;
	}

	/**
	 * @see java.io.InputStream#read()
	 */
	public int read() throws IOException {
		if (offsetInFile == fileDescriptor.length()) {
			// EOF
			return -1;
		}
		if (offsetInSector == currentSector.getSize()) {
			currentSector = fileDescriptor.getNextSector(currentSector);
			offsetInSector = 0;
		}
		currentSector.getBytes(offsetInSector++, singleByteBuffer, 0, 1);
		offsetInFile++;
		return singleByteBuffer[0] & 0xFF;
	}
}

