/*
 * Copyright 2005 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 */
package com.sun.spot.io.j2me.socket;

import java.io.IOException;
import java.io.OutputStream;

/**
 * <p>
 * Socket specific InputSteam. This class handles the passing of protocol information while remaining seamless to the user.
 * </p>
 * @author Martin Morissette
 */
public class SocketProtocolOutputStream extends OutputStream implements SpotSocketProtocol{
    
    private OutputStream out;
    
    /**
     * Create the output stream from another OutputStream.
     * @param out OutputStream to be used
     */
    public SocketProtocolOutputStream(OutputStream out){
        this.out = out;
    }
    
    /**
     * Write data in the stream.
     */
    public void write(int data) throws IOException {
        // If the user of this outputstream wishes to send a character representing a ESC character, 
        // send it with a preceding ESC character.	
        if((data&0xFF)==ESCAPE_CHAR){
            out.write(ESCAPE_CHAR);
        }
        out.write(data);
    }
    
    /**
     * Write an error to the stream.
     * 
     * @throws IOException
     */
    public void writeError(String message) throws IOException{
        out.write(ESCAPE_CHAR);
        out.write(IOEXCEPTION);
        writeProtocolString(message);
        out.flush();
    }

    /**
     * <p>
     * Writes a string within the socket protocol. Protocol strings are used to send extra information.
     * For example, remote exceptions
     * </p>
     * @return String transfered within the socket protocol (invisible to the user)
     * @throws IOException
     */

    private void writeProtocolString(String string) throws IOException{
        
        byte[] bytes = string.getBytes();
        
        int size = bytes.length;
        while (true) {
            int part = ((size > 0x7F)? 0x80:0x00) | (size & 0x7F);
            out.write(part);
            size >>>= 7;
            if (size == 0) {
                break;
            }
        }
        out.write(bytes);
    }
    
    /**
     * Flush the stream. All buffered data will be sent.
     */
    public void flush() throws IOException {
        out.flush();
    }
    
    /**
     * Close the OutputStream
     */
    public void close() throws IOException{
        out.write(ESCAPE_CHAR);
        out.write(CONNECTION_CLOSE);
        out.flush();
        out.close();
        super.close();
    }
    
}
