/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.peripheral;

//import java.io.IOException;
//import java.util.Properties;

import java.io.IOException;
import java.util.Hashtable;

import com.sun.spot.dmamemory.IDMAMemoryManager;
import com.sun.spot.peripheral.ota.OTACommandServer;
import com.sun.spot.peripheral.radio.I802_15_4_MAC;
import com.sun.spot.peripheral.radio.I802_15_4_PHY;
import com.sun.spot.peripheral.radio.IProprietaryRadio;
import com.sun.spot.peripheral.radio.IRadioPolicyManager;
import com.sun.spot.resourcesharing.IResourceRegistry;
import com.sun.spot.util.Properties;


/**
 * The interface to the root object of the Spot base library.<br><br>
 * 
 * This interface provides access to the LED on the Spot board, to the AT91 peripherals,
 * and to other resources such as the software SPI implementation.
 */
public interface ISpot {
	
	/**
	 * Frequency of ARM master clock
	 */
	public static final int MCLK_FREQUENCY = 59904000;

	public static final int SYSTEM_TICKER_TICKS_PER_MILLISECOND = ((MCLK_FREQUENCY  / 128) / 1000);

	/**
	 * System property that if true enables output of log messages on System.err
	 */
	public static final String PROPERTY_SPOT_DIAGNOSTICS = "spot.diagnostics";

	/**
	 * The index used with SpotGlobals to access the globally shared FlashManager
	 */
	public static final int SPOT_GLOBAL_FLASH_MANAGER = 1;
	
	/**
	 * The index used with SpotGlobals to access the globally shared NorFlashSectorFactory
	 */
	public static final int SPOT_GLOBAL_NOR_FLASH_SECTOR_FACTORY = 2;

	/**
	 * @return true if running on the host, false if on the SPOT
	 */
	boolean isRunningOnHost();
	
	/**
	 * @return true if this method has been called in the context of the master isolate
	 */
	boolean isMasterIsolate();

	/**
	 * Get the hardware type code for this device.
	 * @return the hardware type code
	 */
	int getHardwareType();
	
	/**
	 * Get the singleton SpotPins instance.
	 * @return ISpotPins the SpotPins
	 */
	ISpotPins getSpotPins();
	
	/**
	 * Get access to the green LED on the Spot processor board.
	 * Use with caution as this LED is intended for system use.
	 * @return ILed the green LED on the Spot processor board
	 */
	ILed getGreenLed();

	/**
	 * Get access to the red LED on the Spot processor board.
	 * Use with caution as this LED is intended for system use.
	 * @return The red LED on the Spot processor board
	 */
	ILed getRedLed();

	/**
	 * Get the FiqInterruptDaemon. This handles interrupts from the power controller.
	 * @return the FiqInterruptDaemon
	 */
	FiqInterruptDaemon getFiqInterruptDaemon();

	/**
	 * Get access to the AT91 Parallel I/O Controller.
	 * @param pioSelector selects the PIO required (use one of the selector symbols found in IAT91_PIO)
	 * @return the AT91 Parallel Input-Output Controller.
	 */
	IAT91_PIO getAT91_PIO(int pioSelector);

	/**
	 * Get access to the AT91 Interrupt Controller
	 * @return the AT91 Interrupt Controller
	 */
	IAT91_AIC getAT91_AIC();

	/**
	 * Get access to the physical I802.15.4 radio device
	 * @return the I802.15.4 physical radio device
	 */
	I802_15_4_PHY getI802_15_4_PHY();

	/**
	 * Get access to the I802.15.4 MAC layers
	 * @return the I802.15.4 MAC layers
	 */
	I802_15_4_MAC[] getI802_15_4_MACs();

	/**
	 * Get access to the radio via its proprietary (non-I802.15.4) interface.
	 * @return the proprietary interface to the radio device
	 */
	IProprietaryRadio getIProprietaryRadio();

	/**
	 * Get access to the SPI interface.
	 * This interface is used to communicate with the radio and the sensor boards
	 * @return the SPI interface.
	 */
	ISpiMaster getSPI();
	
	/**
	 * Get the Driver Registry.
	 * @return the Driver Registry
	 */
	IDriverRegistry getDriverRegistry();	

	/**
	 * Get the SecuredSiliconArea
	 * @return the SecuredSiliconArea
	 */
	ISecuredSiliconArea getSecuredSiliconArea();
	
	/**
	 * Get access to an AT91 Timer-Counter.
	 * @param index The index of the required TC in the range 0-5
	 * @return The AT91 TC
	 */
	IAT91_TC getAT91_TC(int index);

	/**
	 * Get the configuration page held in flash
	 * @return the configuration page
	 */
	ConfigPage getConfigPage();
	
	/**
	 * Get the Public Key used to sign and verify application and library suites, and the config page. 
	 * @return byte[] The Public Key
	 */
	byte[] getPublicKey();
	
	/**
	 * Write a ConfigPage into the flash.
	 */
	void flashConfigPage(ConfigPage configPage);

	/**
	 * Get the map of external boards. This returns a Hashtable containing an entry for each detected board.
	 * The key of the entry is the PeripheralChipSelect used as the chip select for the board, and the value
	 * is a Properties object containing the properties read from its serial flash memory.
	 * 
	 * @return a Hashtable as described above
	 */
	Hashtable getExternalBoardMap();

	/**
	 * Clear the cache of external board properties. The cache will be re-read from the boards
	 * next time it is used.
	 */
	void resetExternalBoardMap();

	/**
	 * Get access to the flash memory on the Spot
	 * @return the flash memory controller as an IFlashMemoryDevice
	 */
	IFlashMemoryDevice getFlashMemoryDevice();

	/**
	 * Get access to the sleep manager for the Spot
	 * @return The sleep manager
	 */
	ISleepManager getSleepManager();

	/**
	 * power control chip driver
	 * @return the power control chip driver
	 */
	ILTC3455 getLTC3455();
	
	/**
	 * Get the usb power daemon
	 * @return the usb power daemon
	 */
	IUSBPowerDaemon getUsbPowerDaemon();
	
	/**
	 * Get the PowerController - the AVR on the spot.
	 * @return the PowerController
	 */
	IPowerController getPowerController();

	/**
	 * Set a persistent property in the flash memory
	 * @param key
	 * @param value the value required or null to erase
	 */
	void setPersistentProperty(String key, String value);

	/**
	 * Set one or more persistent property in the flash memory
	 * @param props the properties to set
	 */
	void setPersistentProperties(Properties props);

	/**
	 * Replace the persistent properties held in the flash memory
	 * @param props the properties to set
	 * @throws IOException 
	 */
	void storeProperties(Properties props) throws IOException;

	/**
	 * Get a persistent property, as held in the flash memory
	 * @param key
	 * @return the value or null if no property with the specified key
	 */
	String getPersistentProperty(String key);
	
	/**
	 * Get all the persistent properties, as held in the flash memory
	 * @return the set of persistent properties
	 */
	Properties getPersistentProperties();

	/**
	 * Get the OTA Command Server, creating one if necessary
	 * @return the OTA command server
	 * @throws IOException 
	 */
	OTACommandServer getOTACommandServer() throws IOException;

	/**
	 * Get the singleton radio policy manager
	 * @return the radio policy manager
	 */
	IRadioPolicyManager getRadioPolicyManager();

	/**
	 * Set the system property "key" to have the value "value"
	 * 
	 * @param key
	 * @param value
	 */
	void setProperty(String key, String value);

	/**
	 * Return a tick count in the range of zero to {@link #SYSTEM_TICKER_TICKS_PER_MILLISECOND}.
	 * The count resets to zero every millisecond
	 * 
	 * @return the tick count
	 */
	int getSystemTicks();

	/**
	 * Get the singleton resource registry
	 * @return the resource registry
	 */
	IResourceRegistry getResourceRegistry();
	
	/**
	 * Get the singleton AT91 chip power manager
	 * @return the AT91 power manager
	 */
	IAT91_PowerManager getAT91_PowerManager();

	/**
	 * Get the singleton DMA Memory Manager.
	 * The DMA Memory Manager allocates uncached memory suitable for use as DMA buffers.
	 * @return the DMA Memory Manager
	 */
	IDMAMemoryManager getDMAMemoryManager();
}
