/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.peripheral.basestation;

import java.io.IOException;

import com.sun.spot.peripheral.ILed;
import com.sun.spot.peripheral.Spot;
import com.sun.spot.peripheral.SpotFatalException;
import com.sun.spot.peripheral.radio.I802_15_4_MAC;
import com.sun.spot.peripheral.radio.RadioFactory;
import com.sun.spot.peripheral.radio.SpotSerialPipe;
import com.sun.spot.util.Queue;
import com.sun.spot.util.Utils;
import com.sun.squawk.VM;

public class MACProxyServer {
	private Queue commandQueue = new Queue();

	private SpotSerialPipe serialPipe;
	private byte[] inputBuffer = new byte[255];
	private MCPSDataIndicationCommand dataIndicationCommand = new MCPSDataIndicationCommand().with(RadioFactory.getI802_15_4_MAC());
	private MCPSDataRequestCommand dataRequestCommand = new MCPSDataRequestCommand();
	
	ILed receiveLed = Spot.getInstance().getGreenLed();
	ILed sendLed = Spot.getInstance().getRedLed();

	public MACProxyServer(SpotSerialPipe serialPipe) {
		this.serialPipe = serialPipe;
	}

	public void run() {
		Thread ledManagerThread = new Thread(new LedManager(), "Basestation LedManager");
		ledManagerThread.setPriority(Thread.MIN_PRIORITY);
		VM.setAsDaemonThread(ledManagerThread);
		ledManagerThread.start();

		new MACProxyWorkerThread().start();
		new MACProxyWorkerThread().start();
		System.out.println("base station ready ...");
		VM.getCurrentIsolate().clearOut();
		VM.getCurrentIsolate().clearErr();
		VM.getCurrentIsolate().addErr("serial://usart");

		while (true) {
			try {
				commandQueue.put(receiveCommand());
			} catch (Throwable t) {
				System.err.println("[basestation] main thread failed with " + t);
				t.printStackTrace();
			}
		}
	}

	private ICommand receiveCommand() throws IOException {
		serialPipe.receive(inputBuffer);
		
		int uid = Utils.readBigEndInt(inputBuffer, 0);
		byte classIndicatorByte = inputBuffer[Utils.SIZE_OF_INT]; // space for uid
		
		ICommand command = commandFor(classIndicatorByte);
		command.setUID(uid);
		command.populateFrom(inputBuffer, Utils.SIZE_OF_INT + 1); // uid, plus 1 byte for the class indicator 

		return command;
	}

	/**
	 * Synchronize this method rather than SpotSerialPipe.send() for performance gain of around 20% (no, really)
	 */
	private synchronized void serialPipeSend(byte[] outputBuffer, int length) {
		serialPipe.send(outputBuffer, length);
	}
	
	private ICommand commandFor(byte classIndicatorByte) {
		switch (classIndicatorByte) {
			case MACCommand.GetNullPacketAfterAckWaitCommand: 		return new GetNullPacketAfterAckWaitCommand();
			case MACCommand.GetChannelAccessFailureCommand: return new GetChannelAccessFailureCommand();
			case MACCommand.GetNoAckCommand: 				return new GetNoAckCommand();
			case MACCommand.GetPLMETransmitCommand: 		return new GetPLMETransmitCommand();
			case MACCommand.GetWrongAckCommand: 			return new GetWrongAckCommand();
			case MACCommand.MCPSDataIndicationCommand: 		return dataIndicationCommand;
			case MACCommand.MCPSDataRequestCommand: 		return dataRequestCommand;
			case MACCommand.MLMEGetCommand: 				return new MLMEGetCommand();
			case MACCommand.MLMEResetCommand: 				return new MLMEResetCommand();
			case MACCommand.MLMERxEnableCommand: 			return new MLMERxEnableCommand();
			case MACCommand.MLMESetCommand: 				return new MLMESetCommand();
			case MACCommand.MLMEStartCommand: 				return new MLMEStartCommand();
			case MACCommand.SetPLMETransmitPowerCommand: 	return new SetPLMETransmitPowerCommand();
			
			case MACCommand.ResetProxyCommand:				return new ResetProxyCommand().with(this);

			default: throw new SpotFatalException("Do not know a command indicated by "+classIndicatorByte);
		}
	}

	private final class MACProxyWorkerThread extends Thread {
		byte[] outputBuffer = new byte[255];
		private I802_15_4_MAC mac = RadioFactory.getI802_15_4_MAC();
		
		public MACProxyWorkerThread() {
		}
		
		public void run() {
			while (true) {
				try {
					ICommand command = (ICommand)commandQueue.get();
					if (command != null) {
						Utils.writeBigEndInt(outputBuffer, SpotSerialPipe.PAYLOAD_OFFSET, command.getUID());
						int dataLength = command.writeResultOnto(outputBuffer, SpotSerialPipe.PAYLOAD_OFFSET + Utils.SIZE_OF_INT, mac );
						if (!resetting) {
							serialPipeSend(outputBuffer, dataLength + Utils.SIZE_OF_INT); //for the command uid
						}
						if (command == dataRequestCommand) {
							sendLed.setOn(!sendLed.isOn());
						}
						if (command == dataIndicationCommand) {
							receiveLed.setOn(!receiveLed.isOn());
						}
					} else {
						System.err.println("Ignoring null command");
					}
				} catch (Throwable t) {
					System.err.println("[basestation] worker thread failed with " + t);
					t.printStackTrace();
				}
			}
		}
	}

	private boolean resetting = false;

	public void reset(I802_15_4_MAC mac) throws IOException {
		try {
			resetting = true;
			dataIndicationCommand.reset();
			serialPipe.reset();
			Utils.sleep(250); // allow worker threads to clean themselves up
		} finally {
			resetting = false;
		}
	}

	public class LedManager implements Runnable {

		private static final int HEARTBEAT_COUNT = 4;
		private static final long ACTIVITY_DECAY_TIMEOUT = 3000;

		public void run() {
			ILed heartbeatLed = receiveLed;
			while (true) {
				for (int i = 0; i < HEARTBEAT_COUNT; i++) {
					Utils.sleep(ACTIVITY_DECAY_TIMEOUT);
					receiveLed.setOff();
					sendLed.setOff();
				}
				heartbeatLed.setOn();
				Utils.sleep(100);
				heartbeatLed.setOff();
				Utils.sleep(100);
				heartbeatLed.setOn();
				Utils.sleep(100);
				heartbeatLed.setOff();
			}
		}

	}
}
