/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.util;

import java.util.Vector;

/**
 * Implements a fully synchronised FIFO queue of Objects. 
 */
public class Queue {
	private Vector v = new Vector();
	private boolean stopped = false;
	
	/**
	 * Answer whether the receiver is empty.
	 * 
	 * @return - whether the receiver is empty.
	 */
	public boolean isEmpty() {
		synchronized (v) {
			return v.isEmpty();
		}		
	}
	
	/**
	 * Answer the number of elements in the receiver.
	 * 
	 * @return -- the number of elements in the receiver.
	 */
	public int size() {
		synchronized (v) {
			return v.size();
		}		
	}
	
	/**
	 * Add an element to the receiver.
	 *  
	 * @param o -- the Object to add
	 */
	public void put(Object o) {
		synchronized (v) {
			v.addElement(o);
			v.notify();
		}
	}

	/**
	 * Answer an element from the receiver. If the receiver is empty, block
	 * until either an element becomes available, or for timeout milliseconds,
	 * or the queue is stopped. In the event of timeout, return null.
	 * Note that if a waiting thread is interrupted it will return (probably null).
	 * 
	 * @param timeout -- number of milliseconds to wait
	 * @return -- either the first element from the receiver or null after a timeout or if queue is stopped.
	 */
	public Object get(long timeout) {
		synchronized (v) {
			if (timeout > 0 && v.isEmpty() && !stopped) {
				try {
					v.wait(timeout);
				} catch (InterruptedException e) {
					// it's important to catch this exception because we don't notifyAll in
					// put(), so only one thread is unblocked and the thread must consume the
					// element even if it has been interrupted
				}
			}
			return pop();
		}		
	}

	/**
	 * Answer an element from the receiver. If the receiver is empty, block
	 * (possibly forever) until an element becomes available.
	 * Note that if a waiting thread is interrupted it will return (probably null).
	 * 
	 * @return -- the first element from the receiver, or null if queue is stopped
	 */
	public Object get() {
		synchronized (v) {
			while (v.isEmpty() && !stopped) {
				try {
					v.wait();
				} catch (InterruptedException e) {
					// it's important to catch this exception because we don't notifyAll in
					// put(), so only one thread is unblocked and the thread must consume the
					// element even if it has been interrupted
					break;
				}
			}
			return pop();
		}		
	}
	
	/**
	 * Release all waiters
	 */
	public void stop() {
		synchronized (v) {
			stopped = true;
			v.notifyAll();
		}		
	}
	
	/**
	 * Drop the contents of the queue.
	 */
	public void empty() {
		synchronized (v) {
			v.removeAllElements();
		}		
	}

	private Object pop() {
		Object result = null;
		if (!v.isEmpty()) {
			result = v.firstElement();
			v.removeElementAt(0);
		}
		return result;
	}
}
